//@line 37 "/home/ondrejd/mozilla/extensions/xpcomviewer/resources/content/global.js"

/**
 * @version 0.5
 * 
 * This file contains JavaScript code commonly required in the XUL application 
 * development.
 */

//@line 114 "/home/ondrejd/mozilla/extensions/xpcomviewer/resources/content/global.js"


///////////////////////////////////////////////////////////////////////////////
// Constants

const XUL_XMLNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
const XBL_XMLNS = "http://www.mozilla.org/xbl";
const RDF_XMLNS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

const kRdfSrv = Components.classes ["@mozilla.org/rdf/rdf-service;1"].
    getService (Components.interfaces.nsIRDFService);

const kAtomSrv = Components.classes["@mozilla.org/atom-service;1"].
    getService (Components.interfaces.nsIAtomService);

const kPromptSrv = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
    getService (Components.interfaces.nsIPromptService);

const kPrefs = Components.classes ["@mozilla.org/preferences-service;1"].
    getService (Components.interfaces.nsIPrefService).
    getBranch ("extensions.xpcomviewer.");

const kObserverSrv = Components.classes["@mozilla.org/observer-service;1"].
    getService(Components.interfaces.nsIObserverService);
    


///////////////////////////////////////////////////////////////////////////////
// String bundle service for global.js self

/**
 * Object for accessing string bundles needed in global.js file self.
 */
var gGlobalStrBundle = 
{
  str_bundle : null,
  
  initBundle : function()
  {
    try {
      var locsrv = Components.classes["@mozilla.org/intl/nslocaleservice;1"].
          getService(Components.interfaces.nsILocaleService);
      var app_loc = locsrv.GetApplicationLocale();
      var bundlesrv = Components.classes["@mozilla.org/intl/stringbundle;1"].
          getService(Components.interfaces.nsIStringBundleService);
      this.str_bundle = bundlesrv.CreateBundle("chrome://xpcomviewer/locale/global.properties", 
          app_loc);
    } catch(e) {
      Components.utils.reportError(e);
      this.str_bundle = null;
    }    
  },
  
  getString : function(aStringId)
  {
    if(str_bundle == null) this.initBundle();
    
    var ret = "";
    try {
      ret = this.str_bundle.getString(aStringId);
    } catch(e) {
      ret = "";
    }
    return ret;
  }, // end getString(aStringId)
  
  getFormattedString : function(aStringId, aValues)
  {
    if(str_bundle == null) this.initBundle();
    
    var ret = "";
    try {
      ret = this.str_bundle.getFormattedString(aStringId, aValues);
    } catch(e) {
      ret = "";
    }
    return ret;
  } // end getFormattedString(aStringId)
  
}; // end gGlobalStrBundle


///////////////////////////////////////////////////////////////////////////////
// Some enhancements of existing JavaScript String object
// Most of them came from JSLib

/**
 * Capitalize first char of this string
 */
String.prototype.capitalize = function() { 
  return this.charAt (0).toUpperCase () + this.substr (1); 
};

/**
 * Enclose with quotes
 */
String.prototype.withQuotes = function() { 
  return "\"" + this + "\""; 
};

/**
 * Enclose with apostrophes
 */
String.prototype.withApos = function() { 
  return "'" + this + "'"; 
};

/**
 * Check wether this string starts with a specified string
 */
String.prototype.startsWith = function(aStart) { 
  return this.indexOf(aStart) === 0; 
};

/**
 * Check wether this string ends with a specified string
 */
String.prototype.endsWith = function(aEnd) { 
  return this.lastIndexOf(aEnd) == (this.length - aEnd.length); 
};

/**
 * Check wether this string contains a specified string
 */
String.prototype.contains = function(aPattern, aAt) { 
  return this.substr(aAt, aPattern.length) == aPattern; 
};

/**
 * Remove spaces at start
 */
String.prototype.lstrip = function() { 
  return this.replace(/^\s*/, ''); 
};

/**
 * Remove spaces at end
 */
String.prototype.rstrip = function() { 
  return this.replace(/\s*$/, ''); 
};

/**
 * Remove spaces at start and end
 */
var __strip__ = function() { 
  return this.lstrip().rstrip(); 
};
String.prototype.strip = __strip__;
String.prototype.trim = __strip__;

/**
 * Replace withspaces with a specified string
 */
String.prototype.replaceWhitespace = function(aReplaceVal) {
  return this.replace (/(^\s+)|(\s+$)/g, '').replace (/\s+/g, aReplaceVal);
};

/**
 * Split at new line, preserving new line char
 */
String.prototype.splitLines = function(aPreserving) {
  var lineSet = this.split(/\n/g),
  newlineEnding = this.charAt(this.length-1) == '\n';
  
  // Preserve new line char?
  if(aPreserving) {
    for (var i=0,m=lineSet.length-1;i<=m;i++) {
      if((i == m) && !newlineEnding) break;
      lineSet[i] += '\n';
    }
  }
  
  return lineSet;
};

/**
 * Converts string to correct CDATA section
 */
String.prototype.convertToCDATA = function() {
  return this.replace(/\s+/g, "_").replace(/[^a-zA-Z0-9_\.\-\:\u0080-\uFFFF]+/g, '');
};

/**
 * Escape XML special characters
 */
String.prototype.escapeXML = function() {
  return this.replace(/&/g, '&#x26;').replace(/"/g, '&#x22;').
      replace(/'/g, '&#x27;').replace(/</g, '&#x3c;').replace(/>/g, '&#x3e;');
};

/**
 * Unescape XML special characters
 */
String.prototype.unEscapeXML = function() {
  return this.replace(/(&gt;)|(&#x3e;)|(&#x3E;)|(&#62;)/g, '>').
      replace(/(&lt;)|(&#x3c;)|(&#x3C;)|(&#60;)/g, '<').
      replace(/(&apos;)|(&#x27;)|(&#39;)/g, "'").
      replace(/(&quot;)|(&#x22;)|(&#34;)/g, '"').
      replace(/(&amp;)|(&#x26;)|(&#38;)/g, '&');
};

/**
 * XML entity character to JS Unicode convertion
 */
String.prototype.entityToUnicode = function() {
  return this.replace(/&#(x?)([a-fA-F0-9]{2,4});/g,
      function(a,x,d) { return '\\u' + ((d.length==2)?'00':'') + d; });
};

/**
 * Escape RegExp special characters
 */
String.prototype.escapeRegExp = function() {
  return this.replace(/\^/g, '\\^').replace(/\$/g, '\\$').replace(/\+/g, '\\+').
      replace(/\*/g, '\\*').replace(/\./g, '\\.').replace(/\?/g, '\\?').
      replace(/\[/g, '\\[').replace(/\]/g, '\\]');
};

/**
 * Check if this string is a valid email address
 */
String.prototype.isEmailValid = function() { 
  var reg = new RegExp (/^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/); 
  return reg.test (this.trim (this)); 
};

/**
 * Check if this string is a valid url address
 */
String.prototype.isUrlValid = function() { 
  var reg = new RegExp (/(^http:|^ftp:|^https:)/ == -1);
  return reg.test (this.trim (this));  
};



///////////////////////////////////////////////////////////////////////////////
// Two output functions
// XXX Who made this? It's not mine code... correct copyright ...

/**
 * C/C++ sprintf implementation
 */
function sprintf(aFormat,aParameters){
  var argCount = arguments.length;
  var print = '';
  var formatSet = arguments[0].split('%s');
  
  for(var i=1,m=formatSet.length+1;i<m;i++) {
    print += formatSet[i-1] +
        (((i < argCount) && (aParameters[i] != undefined)) ? aParameters[i] : '');
  }
  
  return print;
}

/**
 * Compare specified strings
 */
function strcmp(s1, s2) {
  if((typeof(s1) != 'string') && (typeof(s2) != 'string')) return 1;
  
  var i=0, m=s1.length, n=s2.length;
  while((i<m) && (i<n) && (s1.charAt(i)==s2.charAt(i))) i++;
  
  if(i<m) return (i<n) ? (s1.charCodeAt(i) - s2.charCodeAt(i)) : 1;
  
  return (m==n) ? 0 : -1;
}



///////////////////////////////////////////////////////////////////////////////
// Some enhancements of existing JavaScript Date object

/**
 * Returns current timestamp (I'm using it fore timestamp field in databases, 
 * returned format is this: 8/28/2006 1:47:17 PM
 * @return  string
 */
Date.prototype.getCurrentTimestamp = function() {
  return this.getUTCMonth() + "/" + this.getUTCDate() + "/" +
      this.getUTCFullYear() + " " + this.toLocaleTimeString();
};



///////////////////////////////////////////////////////////////////////////////
// Common dialogs

/**  Show about dialog. */
function showAboutDialog() 
{
  var dlg = window.openDialog("chrome://xpcomviewer/content/about.xul",
      "xpcomviewer-dialog", 
      "chrome,extrachrome,dialog,modal,resizable=no,centerscreen");

  dlg.focus();
}

/**
 * Open JavaScript console
 * @see mozilla/browser/base/content/browser.js
 */
function toJavaScriptConsole()
{
  toOpenWindowByType("global:console", "chrome://global/content/console.xul");
}

/**
 * Open window by type (mozilla app)
 * @see mozilla/browser/base/content/browser.js
 */
function toOpenWindowByType(inType, uri, features)
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var topWindow = windowManagerInterface.getMostRecentWindow(inType);

  if (topWindow)
    topWindow.focus();
  else if (features)
    window.open(uri, "_blank", features);
  else
    window.open(uri, "_blank", "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar");
}

/**
 * Open browser window
 * @see mozilla/browser/base/content/browser.js
 */
function openBrowserWindow()
{
  var charsetArg = new String();
  var handler = Components.classes["@mozilla.org/browser/clh;1"].
      getService(Components.interfaces.nsIBrowserHandler);
  var defaultArgs = handler.defaultArgs;
  var wintype = document.documentElement.getAttribute("windowtype");

  // if and only if the current window is a browser window and it has 
  // a document with a character set, then extract the current charset 
  // menu setting from the current document and use it to initialize 
  // the new browser window...
  var win;
  if (window && (wintype == "navigator:browser") && window.content && 
      window.content.document)
  {
    var DocCharset = window.content.document.characterSet;
    charsetArg = "charset="+DocCharset;

    //we should "inherit" the charset menu setting in a new window
    win = window.openDialog("chrome://browser/content/", "_blank", 
        "chrome,all,dialog=no", defaultArgs, charsetArg);
  }
  else // forget about the charset information.
  {
    win = window.openDialog("chrome://browser/content/", "_blank", 
        "chrome,all,dialog=no", defaultArgs);
  }

  return win;
}

/** 
 * Show preferences dialog
 */
function showPreferences() 
{
  var dlg = window.openDialog("chrome://xpcomviewer/content/preferences.xul",
	    "xpcomviewer-prefs", "chrome,extrachrome,toolbar,modal,resizable=no,centerscreen,all");

  dlg.focus();
}

/** 
 * Show application's help
 */
function showHelp() 
{
  openHelp("xpcomviewer-help", "chrome://xpcomviewer/locale/help/help.rdf");
}

/**
 * Show application's home page
 * Note: Reqire key in extension's preferences file!
 */
function showHomePage()
{
  var extProtocolSvc = Components.classes ["@mozilla.org/uriloader/external-protocol-service;1"].
      getService(Components.interfaces.nsIExternalProtocolService);

  if(extProtocolSvc) 
    extProtocolSvc.loadUrl(createUri(kPrefs.getCharPref("global.homepageUri")));
}



//////////////////////////////////////////////////////////////////////////
// Prompt utilities

/**
 * Utility object for prompting service
 */
var gPromptUtils = 
{
  /**
   * Show alert|warning|error box with given message
   */
  showBox : function(aTitle, aMsg)
  {
    aTitle = this.get_localized_title(aTitle);
    kPromptSrv.alert(window, aTitle, aMsg);
  }, // end showAlert(aTitle, aMsg)
  
  /**
   * Show confirm with given message
   */
  showConfirm : function(aTitle, aMsg, aOkBtnText, aCancelBtnText)
  {
    aTitle = this.get_localized_title(aTitle);
    
    var okFlag = aOkBtnText ? promptService.BUTTON_TITLE_IS_STRING 
                            : promptService.BUTTON_TITLE_OK;
    var cancelFlag = aCancelBtnText ? promptService.BUTTON_TITLE_IS_STRING 
                                    : promptService.BUTTON_TITLE_CANCEL;
    var flags = (okFlag * promptService.BUTTON_POS_0) + 
                (cancelFlag * promptService.BUTTON_POS_1); 
    
    return this.mPromptService.confirmEx(window, aTitle, aMsg, 
        flags, aOkBtnText, aCancelBtnText, null, null, {value:0}) === 0;
  }, // end showConfirm(aTitle, aMsg, aParentWindow)
  
  /**
   * Private method for getting correct string bundles for titles of dialogs
   * @param  string  Dialog's type identifier
   * @return string  Localized string
   */
  get_localized_title : function(aTitle)
  {
    switch(aTitle) 
    {
      case "alert": 
        return gGlobalStrBundle.getString("alertBoxTitle"); 
        break;
      
      case "error": 
        return gGlobalStrBundle.getString("errorBoxTitle"); 
        break;
      
      case "warning":
      default: 
        return gGlobalStrBundle.getString("warningBoxTitle"); 
        break;
    }
    
    return "";
  } // end get_localized_title(aTitle)
  
}; // end of gPromptUtils



//////////////////////////////////////////////////////////////////////////
// IO functions


/**
 * Create file
 * @param  nsIFile
 * @return boolean
 */
function createFile(aFile)
{
  if(!(aFile instanceof Components.interfaces.nsIFile)) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("createFileError1"));
    return false;
  }

  try {
    aFile.create(0x00, 0664);
    return true;
  } catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getFormattedString("createFileError2", [aFile]));
    return false;
  }
} // end createFile(aFile)


/**
 * Create folder
 * @param  nsIFile
 * @return boolean
 */
function createFolder(aFolder)
{
  if (!(aFolder instanceof Components.interfaces.nsIFile)) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("createFolderError1"));
    return false;
  }

  try {
    aFolder.create(0x001, 0664);
    return true;
  } catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getFormattedString("createFolderError2", [aFolder]));
    return false;
  }
} // end createFolder(aFolder)


/**
 * Reads and returns text from file
 * @param  nsILocalFile
 * @return mixed        Returns file's text or FALSE
 */
function readTextFromFile(aFile)
{
  if(!(aFile instanceof Components.interfaces.nsIFile)) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("readTextError1"));
    return false;
  }
  
  var file = Components.classes["@mozilla.org/file/local;1"].
      createInstance(Components.interfaces.nsILocalFile);
  var text = new String ();
  var stream1 = Components.classes["@mozilla.org/network/file-input-stream;1"].
      createInstance(Components.interfaces.nsIFileInputStream);
  var stream2 = Components.classes["@mozilla.org/scriptableinputstream;1"].
      createInstance(Components.interfaces.nsIScriptableInputStream);

  try {
    file.initWithFile(aFile);
    stream1.init(file, 1, 0, false);
    stream2.init(stream1);
    text += stream2.read(-1);

    if(stream2) stream2.close();
    if(stream1) stream1.close();
  } catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getFormattedString("readTextError2", [aFile]));
    return false;
  }

  try {
    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
        createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    converter.charset = "UTF-8";

    return converter.ConvertToUnicode(text);
  } catch(e) {
    Components.utils.reportError(e);
    return text;
  }
} // end readTextFromFile(aFile)


/**
 * Save given text into file
 * @param  nsIFile
 * @param  string
 * @return boolean
 */
function writeTextToFile(aFile, aText)
{
  if(!(aFile instanceof Components.interfaces.nsIFile)) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("saveTextError1"));
    return false;
  }
  
  if(!(aFile instanceof Components.interfaces.nsIFile)) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("saveTextError2"));
    return false;
  }
  
  var file = Components.classes["@mozilla.org/file/local;1"].
      createInstance(Components.interfaces.nsILocalFile);
  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].
      createInstance(Components.interfaces.nsIFileOutputStream);
  var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
      createInstance(Components.interfaces.nsIConverterOutputStream);
  os.init(stream, "UTF-8", 0, 0x0000);

  try {
    file.initWithFile(aFile);
    
    if(file.exists() == false) createFile(file);
    
    stream.init(file, 0x02 | 0x08 | 0x20, 0664, 0);
    os.writeString(aText);
    
    try {
      if(stream) stream.close();
      if(os) os.close();
    } catch(e) {
      // There is obviously when closing nsIConverterOutputStream but we
      // don't care about it
    }
    return true;
  } catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getFormattedString("saveTextError3", [aFile]));
    return false;
  }
} // end writeTextToFile(aFile, aText)


/**
 * Read XML from the given file
 * @param  nsIFile
 * @param  string           File's content type: "text/xml","application/xml" 
 *                          or "application/xhtml+xml"
 * @return nsIDOMDocument
 */
function readXmlFromFile(aFile, aType)
{
  if(!(aFile instanceof Components.interfaces.nsIFile)) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("readXMLError1"));
    return false;
  }
  
  var stream = Components.classes["@mozilla.org/network/file-input-stream;1"].
      createInstance(Components.interfaces.nsIFileInputStream);
  var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"].
      createInstance(Components.interfaces.nsIDOMParser);
  var file = Components.classes["@mozilla.org/file/local;1"].
      createInstance(Components.interfaces.nsILocalFile);
  var doc = null;
  
  try {
    file.initWithFile(aFile);
    stream.init(file, 1, 0, false);
    doc = stream2.parseFromStream(stream, "UTF-8", -1, aType);
    stream.close();
  } catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getFormattedString("readXMLError2", [aFile]));
    return false;
  }

  return doc;
} // end readXmlFromFile(aFile, aDocument)


/**
 * Write XML to the file
 * @param  nsIFile
 * @param  nsIDOMDocument
 * @return boolean
 */
function writeXmlToFile(aFile, aDocument)
{ 
  if(!(aFile instanceof Components.interfaces.nsIFile)) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("writeXMLError1"));
    return false;
  }
  
  var serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].
      createInstance(Components.interfaces.nsIDOMSerializer);
  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].
      createInstance(Components.interfaces.nsIFileOutputStream);
  var file = Components.classes["@mozilla.org/file/local;1"].
      createInstance(Components.interfaces.nsILocalFile);
  
  try {
    file.initWithFile(aFile);
    if(!(file.exists())) file.create(0x00, 0664);
	  stream.init(aFile, 0x02 | 0x08 | 0x20, 0664, 0);
	  serializer.serializeToStream(aDocument, stream, "");
	  stream.close();
    return true;
	} catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getFormattedString("writeXMLError2", [aFile]));
    return false;
	}
} // writeXmlToFile(aFile, aDocument)


/**
 * Returns instance of nsIFile with Profile directory
 * 
 * @todo When we building extension with Firefox (for example is same
 *       for all others Mozilla applications) causes this fuction crash
 *       if extension dependens on it because profile dir of extension
 *       is not in user profile dir but in application profile dir
 */
function getProfDir()
{
  try {
    var file = Components.classes["@mozilla.org/file/directory_service;1"].
       getService(Components.interfaces.nsIProperties).
       get("ProfD", Components.interfaces.nsIFile);

    file.append("extensions");
    /**
     * @todo This variable 'INSTALL_EXTENSION_ID' is defined only when we're
     *       building MOZ_XULAPP!
     */
   file.append("{6E1735B7-2283-42b7-B083-6779DA71C5B1}");
//@line 811 "/home/ondrejd/mozilla/extensions/xpcomviewer/resources/content/global.js"
  } catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("getProfileDirError"));
    return false;
  }
  
  return file;
} // end getProfDir()



//////////////////////////////////////////////////////////////////////////
// Misc. functions



/**
 * Create instance of nsIURI
 */
function createUri(aUrlString)
{
  try {
    var ioserv = Components.classes["@mozilla.org/network/io-service;1"].
        getService(Components.interfaces.nsIIOService);
    return ioserv.newURI(aUrlString, null, null);
  } catch(e) {
    Components.utils.reportError(e);
  }

  return null;
} // end createUri(aUrlString)


/**
 * Set up operating system
 * @return string
 */
function getOS()
{
  var platform = navigator.platform.toLowerCase();

  if(platform.indexOf("win") != -1)
    return "Win";
  else if(platform.indexOf("mac") != -1)
    return "Mac";
  else if(platform.indexOf("unix") != -1 ||
          platform.indexOf("linux") != -1 ||
          platform.indexOf("sun") != -1) 
    return "UNIX";

  return "";
} // end getOS()


/**
 * Load additional JavaScript file from given path into specified window.
 *
 * @param string
 * @param nsIDOMXULWindow
 */
function loadExternalJS(aPath, aWindow)
{
  try {
    var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
        getService(Components.interfaces.mozIJSSubScriptLoader);

    loader.loadSubScript(aPath, aWindow);
  } catch(e) {
    Components.utils.reportError(e);
  }
} // end loadExternalJS (aPath)


/**
 * Prototype function for observing of overlays loading
 */
function NewLoadOverlayObserver(aUri) 
{ 
  this.mUri = aType; 
}
NewLoadOverlayObserver.prototype.observe = function(aSubject, aTopic, aData) 
{
  if(aTopic == "xul-overlay-merged") {
    //Components.utils.reportError("Overlay [" + this.mUri + "] loaded.");
  }
};
NewLoadOverlayObserver.prototype.QueryInterface = function(aIID) 
{
  if(aIID.equals(Components.interfaces.nsIObserver) ||
     aIID.equals(Components.interfaces.nsISupports))
    return this;
  
  return null;
};


/**
 * Load overlay by its type (something as a pre-defined overlay)
 * @param string
 */
function loadOverlayFromUri(aUri)
{
  try {
  	document.loadOverlay(aUri, (new NewLoadOverlayObserver(aUri)));
  } catch(e) {
    Components.utils.reportError(e);
  }
} // end loadOverlayFromUri(aUri)


/**
 * Copy given text to the system clipboard
 * @param string
 */
function copyTextToClipboard(aText)
{
  var nsIClipboard = Components.interfaces.nsIClipboard;
  var clip = Components.classes["@mozilla.org/widget/clipboard;1"].
      createInstance(nsIClipboard);
  if(!clip) return false;

  var trans = Components.classes["@mozilla.org/widget/transferable;1"].
      createInstance(Components.interfaces.nsITransferable);
  if(!trans) return false;
  
  try {
    var str = Components.classes["@mozilla.org/supports-string;1"].
        createInstance(Components.interfaces.nsISupportsString);
    str.data = aText;
    
    trans.addDataFlavor("text/unicode");
    trans.setTransferData("text/unicode", str, aText.length*2);
    clip.setData(trans, null, nsIClipboard.kGlobalClipboard);
    
    return true;
  } catch(e) {
    gPromptUtils.showBox("error", gGlobalStrBundle.getString("copyToClipboardError"));
    return false;
  }
} // end copyTextToClipboard(aText)



//////////////////////////////////////////////////////////////////////////
// Toolbars related functions

var gHaveUpdatedToolbarState = false;


/**
 * Update state of menuitems representing single toolbars
 * @param  string  ID of window we using customizeToolbar dialog in
 * @param  nsIDOMElement
 */
function updateToolbarState(aMainWindowID, aToolbarMenuElm)
{
  if(!gHaveUpdatedToolbarState) 
  {
    var mainWindow = document.getElementById(aMainWindowID);
    if (mainWindow.hasAttribute("chromehidden")) 
    {
      gHaveUpdatedToolbarState = true;

      var i;
      for(i=0; i<aToolbarMenuElt.childNodes.length; ++i) 
      {
        try {
	        document.getElementById(aToolbarMenuElt.childNodes[i].
	            getAttribute("observes")).removeAttribute("checked");
	      } catch(e) {}
      }

      var toolbars = document.getElementsByTagName("toolbar");

      // Start i at 1, since we skip the menubar.
      for(i=1; i<toolbars.length; ++i) 
      {
        if(toolbars[i].getAttribute("class").indexOf("chromeclass") != -1)
          toolbars[i].setAttribute("collapsed", "true");
      }
      
      var statusbars = document.getElementsByTagName("statusbar");
      for(i=1; i<statusbars.length; ++i) 
      {
        if(statusbars[i].getAttribute("class").indexOf("chromeclass") != -1)
          statusbars[i].setAttribute("collapsed", "true");
      }
      mainWindow.removeAttribute("chromehidden");
    }
  }
} // end updateToolbarState(aMainWindowID, aToolbarMenuElm)


/**
 * Fired when toolbars popup menu is showing
 * @param nsIDOMEvent
 */
function onViewToolbarsPopupShowing(aEvent)
{
  var popup = aEvent.target;
  var i;

  // Empty the menu
  for(i=popup.childNodes.length-1; i>=0; --i) 
  {
    var deadItem = popup.childNodes[i];
    if(deadItem.hasAttribute("toolbarindex"))
      popup.removeChild(deadItem);
  }

  var firstMenuItem = popup.firstChild;

  var toolbox = document.getElementById("workspace-toolbox");
  for(i=0; i<toolbox.childNodes.length; ++i) 
  {
    var toolbar = toolbox.childNodes[i];
    var toolbarName = toolbar.getAttribute("toolbarname");
    var type = toolbar.getAttribute ("type");
    if (toolbarName && type != "menubar") 
    {
      var menuItem = document.createElementNS(kXUL_XMLNS, "menuitem");
      menuItem.setAttribute("toolbarindex", i);
      menuItem.setAttribute("type", "checkbox");
      menuItem.setAttribute("label", toolbarName);
      menuItem.setAttribute("accesskey", toolbar.getAttribute("accesskey"));
      menuItem.setAttribute("checked", toolbar.getAttribute("collapsed") != "true");
      popup.insertBefore(menuItem, firstMenuItem);
      menuItem.addEventListener("command", onViewToolbarCommand, false);
    }
    toolbar = toolbar.nextSibling;
  }
} // end onViewToolbarsPopupShowing(aEvent)


/**
 * Toggle toolbar view command
 * @param nsIDOMEvent
 */
function onViewToolbarCommand(aEvent)
{
  var toolbox = document.getElementById("workspace-toolbox");
  var index = aEvent.originalTarget.getAttribute("toolbarindex");
  var toolbar = toolbox.childNodes[index];

  toolbar.collapsed = aEvent.originalTarget.getAttribute("checked") != "true";
  document.persist(toolbar.id, "collapsed");
} // end onViewToolbarCommand(aEvent)


/**
 * Show customize toolbar dialog
 * @param  string  ID of customized toolbar
 * @param  string  ID of main menubar
 */
function customizeToolbar(aCustomizeToolbarCmdID, aMainMenubarID)
{
  // Disable the toolbar context menu items
  var menubar = document.getElementById(aMainMenubarID);

  for(var i=0; i<menubar.childNodes.length; ++i) 
  {
    menubar.childNodes[i].setAttribute("disabled", true);
  }

  var cmd = document.getElementById(aCustomizeToolbarCmdID);
  cmd.setAttribute("disabled", "true");

  window.openDialog("chrome://global/content/customizeToolbar.xul",
      "CustomizeToolbar", "chrome,all,dependent",
      [document.getElementById("workspace-toolbox"), aCustomizeToolbarCmdID]);
} // end customizeToolbar(aCustomizeToolbarCmdID, aMainMenubarID)



//////////////////////////////////////////////////////////////////////////
// Tree-related variables and objects


/**
 * This function creates new common tree item for using with our 
 * TreeViewPrototype. As columns pass array of objects with column name 
 * and value; e.g { mColId : "", mColText : ""}. 
 * 
 * @param  array   Array with columns objects
 * @param  integer Level of item
 * @param  boolean Is item container?
 * @return object
 */
function TreeViewItemPrototype(aColumns, aLevel, aIsContainer) 
{
  var item = new Object();
  
  item.mColumns = new Object();
  for(var i=0; i<aColumns.length; i++) 
  {
	  item.mColumns[aColumns[i].mColId] = { mColId		 : aColumns[i].mColId, 
	  																			mCellText  : aColumns[i].mColText };
	}
	
  item.mIsContainer 			= aIsContainer;
  item.mLevel 						= aLevel;
  item.mIsContainerOpen 	= false;
  item.mIsContainerEmpty 	= true;
  item.mHiddenRows 				= [];
  
  return item;
}


/**
 * Common tree view prototype object. As a parameter is expected used 
 * tree view items.
 * @param  array  Array of treeview items 
 */
function TreeViewPrototype(aTreeView) 
{
  this.__treeView__ = aTreeView;  // Array with tree items
}

TreeViewPrototype.prototype.treebox = null;
TreeViewPrototype.prototype.selection = null;

TreeViewPrototype.prototype.__defineGetter__ ("rowCount", 
		function () { return this.__treeView__.length; });

TreeViewPrototype.prototype.getCellText = function(aRow, aCol) 
{
	if (!(aRow in this.__treeView__)) return "";
	if (!(aCol.id in this.__treeView__[aRow].mColumns)) return "";
	
	return this.__treeView__[aRow].mColumns[aCol.id].mCellText;
};

TreeViewPrototype.prototype.getCellValue = function(aRow, aCol) 
{ 
	if (!(aRow in this.__treeView__)) return null;
	if (!(aCol.id in this.__treeView__[aRow].mColumns)) return null;
	
	// XXX Implement this for cells wich are cyclers
	return null;
};

TreeViewPrototype.prototype.getRowProperties = function(aRow, aProp) { return; };
TreeViewPrototype.prototype.getCellProperties = function(aRow, aCol, aProp) { return; };
TreeViewPrototype.prototype.getColumnProperties = function(aCol, aProp) { return; };

TreeViewPrototype.prototype.isContainer = function(aRow) 
{ 
	if (!(aRow in this.__treeView__)) return false;
	
	return this.__treeView__[aRow].mIsContainer;
};

TreeViewPrototype.prototype.isContainerOpen = function(aRow)
{ 
	// Tests if given row is in our tree view are in this::isContainer(aRow)...
	return ((this.isContainer(aRow)))
	    ? this.__treeView__[aRow].mIsContainerOpen
	    : false;
};

TreeViewPrototype.prototype.isContainerEmpty = function(aRow)
{ 
	// Tests if given row is in our tree view are in this::isContainer(aRow)...
	return ((this.isContainer(aRow)))
	    ? this.__treeView__[aRow].mIsContainerEmpty
	    : false;
};

TreeViewPrototype.prototype.canDrop = function(aRow, aOrientation) { return true; };
TreeViewPrototype.prototype.drop = function(aRow, aOrientation) {};

TreeViewPrototype.prototype.setTree = function(aOut) { this.treebox = aOut; };

TreeViewPrototype.prototype.getParentIndex = function(aRow)
{
	if (!(aRow in this.__treeView__)) return -1;

  for(var i=(aRow-1); i>=0; i--) {
    if(this.getLevel(i)<this.getLevel(aRow)) {
    	return i;
    }
  }
	
	return -1;
};

TreeViewPrototype.prototype.hasNextSibling = function(aRow, aAfterIndex)
{
	if (!(aRow in this.__treeView__)) return false;
	
	var thisLevel = this.getLevel(aRow);
	for (var i=(aRow+1); i<this.rowCount; i++) 
	{
	  var nextLevel = this.getLevel(i);
	  
	  if(nextLevel == thisLevel) return true;
	  else if(nextLevel < thisLevel) return false;
  }
  
  return false;
};

TreeViewPrototype.prototype.getLevel = function(aRow) 
{
	if (!(aRow in this.__treeView__)) return -1;
	
	return this.__treeView__[aRow].mLevel;
};

TreeViewPrototype.prototype.getImageSrc = function(aRow, aCol) { return ""; };
TreeViewPrototype.prototype.toggleOpenState = function(aIndex) {};
TreeViewPrototype.prototype.isSorted = function() { return false; };
TreeViewPrototype.prototype.cycleHeader = function(aCol) { return; };
TreeViewPrototype.prototype.selectionChanged = function() {};
TreeViewPrototype.prototype.cycleCell = function(aRow, aCol) {};
TreeViewPrototype.prototype.isEditable = function(aRow, aCol) { return false; };
TreeViewPrototype.prototype.setCellText = function(aRow, aCol, aValue) {};
TreeViewPrototype.prototype.setCellValue = function(aRow, aCol, aValue) {};
TreeViewPrototype.prototype.performAction = function(aAction) {};
TreeViewPrototype.prototype.performActionOnRow = function(aAction, aRow) {};
TreeViewPrototype.prototype.preformActionOnCell = function(aAction, aRow, aCol) {};
TreeViewPrototype.prototype.isSeparator = function(aIndex) { return false; };


//////////////////////////////////////////////////////////////////////////
// Prototype objects for nsITimer and timer observer

/**
 * Prototype object for timer observer
 */
function TimerObserverPrototype(aCallback)
{
  // Implementation of Components.interfaces.nsIObserver
  this.observe = aCallback;
}

// Implementation of Components.interfaces.nsISupports
TimerObserverPrototype.prototype.QueryInterface = function(aIID)
{
  if(aIID.equals(Components.interfaces.nsIObserver) || 
     aIID.equals(Components.interfaces.nsISupportsWeakReference) || 
     aIID.equals(Components.interfaces.nsISupports))
  {
    return this;
  }
  
  throw Components.results.NS_NOINTERFACE;
};


/**
 * Prototype object for timer self
 */
function TimerPrototype()
{
  this.mTimer = Components.classes["@mozilla.org/timer;1"].
      createInstance(Components.interfaces.nsITimer);
  
  // Shortcuts for timer types
  this.TYPE_REPEATING_PRECISE = Components.interfaces.nsITimer.TYPE_REPEATING_PRECISE;
  this.TYPE_REPEATING_SLACK = Components.interfaces.nsITimer.TYPE_REPEATING_SLACK;
  this.TYPE_ONE_SHOT = Components.interfaces.nsITimer.TYPE_ONE_SHOT;
}

/**
 * Initialize timer
 * @param object Callback function
 * @param int    Time in miliseconds
 * @param int    Type of timer [TYPE_REPEATING_PRECISE|TYPE_REPEATING_SLACK|TYPE_ONE_SHOT]
 */
TimerPrototype.prototype.init = function(aCallback, aTime, aType)
{
  var observer = new TimerObserverPrototype(aCallback);
  this.mTimer.init(observer, aTime,
                   ((aType === null) ? this.TYPE_ONE_SHOT : aType));
};
